/*!
  * vue-promised v0.0.0
  * (c) 2020 Eduardo San Martin Morote
  * @license MIT
  */
var VuePromised = (function (exports, vue) {
  'use strict';

  /**
   * Returns the state of a Promise and observes the Promise if it's a Ref to
   * automatically update the state
   *
   * @param promise Ref of a Promise or raw Promise
   * @param pendingDelay optional delay to wait before displaying pending
   */
  function usePromise(promise, pendingDelay = 200) {
      const isPending = vue.ref(false);
      const isDelayElapsed = vue.ref(false);
      const error = vue.ref();
      const data = vue.ref();
      let timerId;
      vue.watch(() => vue.unref(promise), (newPromise) => {
          isPending.value = true;
          error.value = null;
          if (!newPromise) {
              data.value = null;
              isPending.value = true;
              if (timerId)
                  clearTimeout(timerId);
              timerId = null;
              return;
          }
          if (vue.unref(pendingDelay) > 0) {
              isDelayElapsed.value = false;
              if (timerId)
                  clearTimeout(timerId);
              timerId = setTimeout(() => {
                  isDelayElapsed.value = true;
              }, Number(vue.unref(pendingDelay)));
          }
          else {
              isDelayElapsed.value = true;
          }
          newPromise
              .then((newData) => {
              // ensure we are dealing with the same promise
              if (newPromise === vue.unref(promise)) {
                  data.value = newData;
                  isPending.value = false;
              }
          })
              .catch((err) => {
              // ensure we are dealing with the same promise
              if (newPromise === vue.unref(promise)) {
                  error.value = err;
                  isPending.value = false;
              }
          });
      }, { immediate: true });
      return { isPending, isDelayElapsed, error, data };
  }

  const Promised = vue.defineComponent({
      name: 'Promised',
      props: {
          promise: {},
          // validator: p =>
          //   p && typeof (p as any).then === 'function' && typeof (p as any).catch === 'function',
          pendingDelay: {
              type: [Number, String],
              default: 200,
          },
      },
      setup(props, { slots }) {
          const propsAsRefs = vue.toRefs(props);
          const promiseState = vue.reactive(usePromise(propsAsRefs.promise, propsAsRefs.pendingDelay));
          return () => {
              if ('combined' in slots) {
                  return slots.combined(promiseState);
              }
              const [slotName, slotData] = promiseState.error
                  ? ['rejected', promiseState.error]
                  : !promiseState.isPending
                      ? ['default', promiseState.data]
                      : promiseState.isDelayElapsed
                          ? ['pending', promiseState.data]
                          : [null];
              if ( slotName && !slots[slotName]) {
                  vue.warn(`[vue-promised]: Missing slot "${slotName}". This will fail in production.`);
                  return null;
              }
              return slotName && slots[slotName](slotData);
          };
      },
  });

  exports.Promised = Promised;
  exports.usePromise = usePromise;

  Object.defineProperty(exports, '__esModule', { value: true });

  return exports;

}({}, Vue));
